<?php

namespace Emp\Employee\Policies;

use Litepie\User\Contracts\UserPolicy;
use Emp\Employee\Models\Employee;

class EmployeePolicy
{

    /**
     * Determine if the given user can view the employee.
     *
     * @param UserPolicy $user
     * @param Employee $employee
     *
     * @return bool
     */
    public function view(UserPolicy $user, Employee $employee)
    {
        if ($user->canDo('employee.employee.view') && $user->isAdmin()) {
            return true;
        }

        return $employee->user_id == user_id() && $employee->user_type == user_type();
    }

    /**
     * Determine if the given user can create a employee.
     *
     * @param UserPolicy $user
     * @param Employee $employee
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('employee.employee.create');
    }

    /**
     * Determine if the given user can update the given employee.
     *
     * @param UserPolicy $user
     * @param Employee $employee
     *
     * @return bool
     */
    public function update(UserPolicy $user, Employee $employee)
    {
        if ($user->canDo('employee.employee.edit') && $user->isAdmin()) {
            return true;
        }

        return $employee->user_id == user_id() && $employee->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given employee.
     *
     * @param UserPolicy $user
     * @param Employee $employee
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Employee $employee)
    {
        return $employee->user_id == user_id() && $employee->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given employee.
     *
     * @param UserPolicy $user
     * @param Employee $employee
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Employee $employee)
    {
        if ($user->canDo('employee.employee.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given employee.
     *
     * @param UserPolicy $user
     * @param Employee $employee
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Employee $employee)
    {
        if ($user->canDo('employee.employee.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
